import { readFile } from 'node:fs/promises';
import matter from 'gray-matter';
import { parse } from 'marked';
import { buildFile, getStylePath } from './utils';
import { format } from 'prettier';
import Root from './root';
import { writeFileSync } from 'node:fs';
import { resolve } from 'path';
import crypto from 'node:crypto';

export const buildBlog = async (blog: string) => {
  const fileContent = await readFile(blog, 'utf-8');
  const content = matter(fileContent);
  const postHtml = parse(content.content);

  const sha = crypto.createHash('sha1').update(fileContent).digest('hex');

  const templateFile = content.data.template;
  const templatePath = resolve(`./src/templates/${templateFile}.tsx`);

  const blogFileName = blog.replace('src/blog/', '').replace('.md', '');

  const tmpJsPath = templatePath
    .replace('src', '.tmp')
    .replace(content.data.template, sha)
    .replace('.tsx', '.js');

  const blogOutputHtmlPath = templatePath
    .replace('src/templates', '.out')
    .replace(content.data.template, blogFileName)
    .replace('.tsx', '.html');

  await buildFile(templatePath, tmpJsPath);

  const { default: Page } = await import(tmpJsPath);
  const html = Page.default({ children: postHtml });
  let stylePaths: string[] = getStylePath(tmpJsPath, blogOutputHtmlPath);

  const formattedHtml = await format(
    <Root title="My App" stylePaths={stylePaths} scriptPaths={[]}>
      {html}
    </Root>,
    { parser: 'html' },
  );

  writeFileSync(blogOutputHtmlPath, formattedHtml);
};
