import { cpSync, existsSync } from 'node:fs';
import { resolve } from 'path';
import { sassPlugin } from 'esbuild-sass-plugin';
import { build } from 'esbuild';

export const outDir = resolve('.out');

export const getStylePath = (tmpJsPath: string, pageOutputHtmlPath: string) => {
  let stylePaths: string[] = [];

  if (existsSync(tmpJsPath.replace('.js', '.css'))) {
    cpSync(
      tmpJsPath.replace('.js', '.css'),
      pageOutputHtmlPath.replace('.html', '.css'),
    );

    stylePaths.push(
      pageOutputHtmlPath.replace('.html', '.css').replace(outDir, ''),
    );
  }

  return stylePaths;
};

export const getPagePaths = (rawPath: string) => {
  const pagePath = resolve(rawPath);
  const tmpJsPath = pagePath.replace('src', '.tmp').replace('.tsx', '.js');
  const pageOutputHtmlPath = pagePath
    .replace('src/pages', '.out')
    .replace('.tsx', '.html');
  const blogOutputHtmlPath = pagePath
    .replace('src/templates', '.out')
    .replace('.tsx', '.html');

  return { pagePath, tmpJsPath, pageOutputHtmlPath, blogOutputHtmlPath };
};

export const buildFile = async (input: string, output: string) =>
  await build({
    entryPoints: [input],
    bundle: true,
    outfile: output,
    jsxImportSource: '@kitajs/html',
    minify: true,
    platform: 'node',
    external: ['esbuild'],
    plugins: [
      sassPlugin({
        filter: /\.module\.scss$/,
        type: 'local-css',
      }),
      sassPlugin({
        filter: /\.scss$/,
        type: 'css',
      }),
    ],
  });
